<?php
namespace plugins\surveys\models;

use \library\Orderable;
use \ExtraColumnsObserver;
use \observers\OrderableObserver;

/**
 * Question model class
 * @author Jérôme Brilland
 * @version 1.3
 * @copyright (c) 2022, Jérôme Brilland
 * @license http://www.gnu.org/licenses/gpl.txt GNU General Public License
 */
class Question extends \ActiveRecord {
    
    use Orderable;

    protected $types = ['single_choice', 'multiple_choice', 'short_answer', 'long_answer', 'email', 'numeric', 'likert_scale', 'ranking', 'date', 'time'];

    private $choices = [];

    protected function initialize() {
        $this->attach( new ExtraColumnsObserver( 'properties' ) );
        $this->attach( new OrderableObserver() );
    }

    protected function validate() {
        $i18n = \IoC::resolve( 'i18n' );
        $this->validateMaxLength( 'text', 255, $i18n->__( 'surveys.too_long_text', 255 ) );
        $this->validateNotEmpty( 'text', $i18n->__( 'surveys.text_cannot_be_empty' ) );
        $this->validateInArray( 'type', $this->types, $i18n->__( 'surveys.invalid_type' ) );
        if ( 'short_answer' == $this->type || 'long_answer' == $this->type ) {
            $this->validateInteger( 'maxlength', 'Maxlength must be an integer.' );
            if ( !empty( $this->minlength ) ) {
                $this->validateInteger( 'minlength', 'Minlength must be an integer.' );
            }
            $this->validateMin( 'maxlength', $this->minlength ?: 0, $i18n->__('surveys.maxlength_greater_than_or_equal_to_minlength') ) ;
        } else if ( 'multiple_choice' == $this->type ) {
            if ( !empty( $this->max_choices ) ) {
                $this->validateInteger( 'max_choices', 'Maximum choices must be an integer.' );
                $this->validateMin( 'max_choices', $this->min_choices ?: 0, $i18n->__('surveys.max_choices_greater_than_or_equal_to', $this->min_choices ?: 0 ) ) ;
                $this->validateMax( 'max_choices', count( $this->choices ), $i18n->__( 'surveys.max_choices_less_than_or_equal_to', count( $this->choices ) ) );
            }
            if ( !empty( $this->min_choices ) ) {
                $this->validateInteger( 'min_choices', 'Minimum choices must be an integer.' );
                $this->validateMin( 'min_choices', 0, $i18n->__( 'surveys.min_choices_greater_than_or_equal_to', 0 ) );
            }
        } else if ( 'numeric' == $this->type ) {
            if ( !empty( $this->max ) ) {
                $this->validateNumber( 'max', 'Maximum must be a number.' );
                if ( !empty( $this->min ) ) {
                    $this->validateMin( 'max', $this->min, $i18n->__('surveys.max_greater_than_or_equal_to_min' ) ) ;
                }
            }
            if ( !empty( $this->min ) ) {
                $this->validateNumber( 'min', 'Minimum must be a number.' );
            }
        }
        
        foreach( $this->choices as $choice ) {
            if ( $choice->isInvalid() && !array_key_exists( 'choices', $this->getErrors() ) ) {
                $this->addError( 'choices', true );
            }
        }
    }

    protected function beforeSave() {
        $this->text = trim( $this->text );
    }

    protected function afterSave() {
        foreach( $this->choices as $choice ) {
            if ( $choice->_delete ) {
                if ( !$choice->isNewRecord() ) {
                    $choice->delete();
                }
            } else {
                $choice->question_id = $this->question_id;
                $choice->save();
            }
        }
    }

    protected function getChoices() {
        return $this->choices ?: $this->choices = Choice::findMany( '*', ['question_id' => $this->question_id] );
    }
    
    protected function setChoiceAttributes( $choiceAttributes ) {
        foreach( $choiceAttributes as $attributes ) {
            $this->choices[] = Choice::findOrNew( ['choice_id' => $attributes['choice_id'] ?? null, 'question_id' => $this->question_id], $attributes ) ;
        }
    }
    
    public function addChoice( $choice ) {
		$this->choices[] = $choice;
	}
}
