<?php
namespace plugins\surveys\models;

use plugins\surveys\models\Question;
/**
 * Survey model class
 * @author Jérôme Brilland
 * @version 1.1
 * @copyright (c) 2022, Jérôme Brilland
 * @license http://www.gnu.org/licenses/gpl.txt GNU General Public License
 */
class Survey extends \ActiveRecord {

    const INACTIVE = 0;
    const ACTIVE = 1;

    private $questions = [];

    protected function initialize() {
        $this->attach( new \NullifyBlankObserver() );

        if ( !isset( $this->start_date ) ) {
            $this->start_date = date( 'Y-m-d' );
        }
        if ( !isset( $this->start_time ) ) {
            $this->start_time = date( 'H:i:s' );
        }
    }

    protected function validate() {
        $i18n = \IoC::resolve( 'i18n' );
        $this->validateMaxLength( 'title', 255, $i18n->__( 'too_long_title', 255 ) );
        $this->validateNotEmpty( 'title', $i18n->__( 'title_cannot_be_empty' ) );
        $this->validateMaxLength( 'language', 2, $i18n->__( 'too_long_language', 2 ) );
        $this->validateDate( 'start_date', $i18n->__( 'surveys.invalid_start_date' ) );
        $this->validateNotEmpty( 'start_date', $i18n->__( 'surveys.start_date_is_required' ) );
        $this->validateTime( 'start_time', $i18n->__( 'surveys.invalid_start_time' ) );
        $this->validateNotEmpty( 'start_time', $i18n->__( 'surveys.start_time_is_required' ) );
        if ( !empty( $this->end_date ) ) {
            $this->validateNotEmpty( 'end_time', $i18n->__( 'surveys.end_time_is_required' ) );

            $startDate = new \DateTimeImmutable( $this->start_date . ' ' . $this->start_time );
            $endDate = new \DateTimeImmutable( $this->end_date . ' ' . $this->end_time );
            if ( $endDate <= $startDate ) {
                $this->addError( 'end_date', $i18n->__( 'surveys.end_after_start' ) );
            }
        }
        $this->validateInArray( 'status', [self::INACTIVE, self::ACTIVE], $i18n->__( 'invalid_status' ) );
        
        foreach( $this->questions as $question ) {
            if ( $question->isInvalid() && !array_key_exists( 'questions', $this->getErrors() ) ) {
                $this->addError( 'questions', true );
            }
        }
    }

    protected function afterFind() {
        if ( empty( $this->description ) ) {
            $this->description = '';
        }
    }
    
    protected function beforeInsert() {
        $this->user_id = \IoC::resolve( 'user' )->user_id;
    }

    protected function beforeSave() {
        $this->title = trim( $this->title );
        $this->description = trim( $this->description );
    }

    protected function afterSave() {
        foreach ( $this->questions as $question ) {
            if ( $question->_delete ) {
                if ( !$question->isNewRecord() ) {
                    $question->delete();
                }
            } else {
                $question->survey_id = $this->survey_id;
                $question->save();
            }
        }
    }

    protected function getQuestions() {
        return $this->questions ?: $this->questions = Question::findMany( '*', ['survey_id' => $this->survey_id, 'ORDER' => ['position' => 'ASC']] );
    }

    protected function setQuestionAttributes( $questionAttributes ) {
        foreach( $questionAttributes as $attributes ) {
            $this->questions[] = Question::findOrNew( ['question_id' => $attributes['question_id'] ?? null, 'survey_id' => $this->survey_id], $attributes );
        }
    }

    public function isOpen() {
        $now = new \DateTimeImmutable();
        $startDate = new \DateTimeImmutable( $this->start_date . ' ' . $this->start_time );
        $endDate = ( !empty( $this->end_date ) ) ? new \DateTimeImmutable( $this->end_date . ' ' . $this->end_time ) : null;
        return ( self::ACTIVE == $this->status && $startDate <= $now ) && ( empty( $endDate ) || ( $endDate > $now ) );
    }

    public function isClosed() {
        return !$this->isOpen();
    }

    public static function findOpenSurveys() {
        $columns = [
            'survey_id',
            'title',
            'description',
            'language',
            'user_id',
            'start_date',
            'start_time',
            'end_date',
            'end_time'
        ];
        $where = ['status' => self::ACTIVE,
            'OR #first' => ['start_date[<]' => date( 'Y-m-d' ),
                'AND' => ['start_date' => date( 'Y-m-d' ), 'start_time[<=]' => date( 'H:i:s' )]],
            'OR #second' => ['end_date' => null,
                'end_date[>]' => date( 'Y-m-d' ),
                'AND' => ['end_date' => date( 'Y-m-d' ), 'end_time[>]' => date( 'H:i:s' )]],
            'ORDER' => ['start_date', 'start_time']];
        return Survey::findMany( $columns, $where );
    }
}
